import json
import logging
import os
import sys

import yaml

from explaining_framework.config.explaining_config import explaining_cfg


def read_json(path: str) -> dict:
    with open(path, "r") as f:
        data = json.load(f)
    return data


def write_json(data: dict, path: str) -> None:
    with open(path, "w") as f:
        data = json.dump(data, f)


def read_yaml(path: str) -> dict:
    with open(path, "r") as f:
        data = yaml.safe_load(f)
    return data


def write_yaml(data: dict, path: str) -> None:
    with open(path, "w") as f:
        data = yaml.dump(data, f)


def dump_cfg(cfg, path):
    r"""
    Dumps the config to the output directory specified in
    :obj:`cfg.out_dir`
    Args:
        cfg (CfgNode): Configuration node
    """
    with open(path, "w") as f:
        cfg.dump(stream=f)


def is_exists(path: str) -> bool:
    return os.path.exists(path)


def get_dict_config(d: dict):
    config = {}
    for k, v in d.items():
        if isinstance(v, (int, float, str, bool)):
            config[k] = val_check(v)
    return config


def val_check(v):
    if v == float("-inf"):
        return "minus_inf"
    else:
        return v


def save_obj_config(obj, path) -> None:
    config = get_obj_config(obj)
    write_json(config, path)


def obj_config_to_str(obj) -> str:
    if isinstance(obj, dict):
        config = get_dict_config(obj)
        return "-".join([f"{k}={v}" for k, v in config.items()])
    else:
        config = get_dict_config(obj.__dict__)
        return "-".join([f"{k}={v}" for k, v in config.items()])


def obj_config_to_log(obj) -> str:
    if isinstance(obj, dict):
        config = get_dict_config(obj)
        for k, v in config.items():
            logging.info(f"{k} : {v}")
    else:
        config = get_dict_config(obj.__dict__)
        for k, v in config.items():
            logging.info(f"{k} : {v}")


def set_printing(logger_path):
    """
    Set up printing options

    """
    logging.root.handlers = []
    logging_cfg = {
        "level": logging.INFO,
        "format": "%(asctime)s:%(levelname)s:%(message)s",
    }
    h_file = logging.FileHandler(logger_path)
    h_stdout = logging.StreamHandler(sys.stdout)
    if explaining_cfg.print == "file":
        logging_cfg["handlers"] = [h_file]
    elif explaining_cfg.print == "stdout":
        logging_cfg["handlers"] = [h_stdout]
    elif explaining_cfg.print == "both":
        logging_cfg["handlers"] = [h_file, h_stdout]
    else:
        raise ValueError("Print option not supported")
    logging.basicConfig(**logging_cfg)
